#!/usr/bin/python
# -*- coding: utf-8 -*-
#################################################################################
# LAYMAN TAR OVERLAY HANDLER
#################################################################################
# File:       tar.py
#
#             Handles tar overlays
#
# Copyright:
#             (c) 2005 - 2006 Gunnar Wrobel
#             Distributed under the terms of the GNU General Public License v2
#
# Author(s):
#             Gunnar Wrobel <wrobel@gentoo.org>
#

__version__ = "$Id: tar.py 168 2006-06-26 22:37:29Z wrobel $"

#===============================================================================
#
# Dependencies
#
#-------------------------------------------------------------------------------

import os, os.path, sys, urllib, shutil

from   layman.utils             import path
from   layman.debug             import OUT
from   layman.overlays.overlay  import Overlay

#===============================================================================
#
# Class TarOverlay
#
#-------------------------------------------------------------------------------

class TarOverlay(Overlay):
    ''' Handles tar overlays.

    A dummy tar handler that overwrites the __init__ method
    so that we don't need to provide xml input:

    >>> class DummyTar(TarOverlay):
    ...   def __init__(self):
    ...     self.name = 'dummy'
    ...     here = os.path.dirname(os.path.realpath(__file__))
    ...     self.src  = here + '/../tests/testfiles/layman-test.tar.bz2'
    ...     self.subpath = 'layman-test'

    >>> testdir = os.tmpnam()
    >>> os.mkdir(testdir)
    >>> a = DummyTar()
    >>> OUT.color_off()
    >>> a.add(testdir) #doctest: +ELLIPSIS
    * Running command "/bin/tar -v -x -j -f...
    >>> sorted(os.listdir(testdir + '/dummy'))
    ['app-admin', 'app-portage']
    >>> shutil.rmtree(testdir)
    '''

    type = 'Tar'
    
    binary = '/bin/tar' 

    def __init__(self, xml):
        
        Overlay.__init__(self, xml)

        if '&subpath' in self.data.keys():
            self.subpath = self.data['&subpath']
        else:
            self.subpath = ''
        
        if '&category' in self.data.keys():
            if self.subpath:
                raise Exception('Cannot use "category" and "subpath" at the same'
                                ' time!')

            self.category = self.data['&category']
        else:
            self.category = ''
        
    def add(self, base):

        self.supported()

        dir = path([base, self.name])

        if os.path.exists(dir):
            raise Exception('Directory ' + dir + ' already exists. Will not ove'
                            'rwrite its contents!')

        if self.src[-3:] == 'bz2':
            ext = 'bz2'
            opt = '-j'
        elif self.src[-2:] == 'gz':
            ext = 'gz'
            opt = '-z'

        try:

            tar = urllib.urlopen(self.src).read()

        except Exception, e:
            raise Exception('Failed to fetch the tar package from: ' 
                            + self.src + '\nError was:' + str(e))

        pkg = path([base, self.name + '.tar.' + ext])

        try:

            out_file = open(pkg, 'w')
            out_file.write(tar)
            out_file.close()

        except Exception, e:
            raise Exception('Failed to store tar package in ' 
                            + pkg + '\nError was:' + str(e))

        if self.subpath:
            target = path([base, 'tmp'])
        else:
            if self.category:
                target = dir + '/' + self.category
            else:
                target = dir

        os.makedirs(target)

        result = self.cmd(self.binary + ' -v -x ' + opt + ' -f ' + pkg + ' -C ' 
                          + target)

        if self.subpath:
            source = target + '/' + self.subpath
            if os.path.exists(source):
                try:
                    os.rename(source, dir)
                except Exception, e:
                    raise Exception('Failed to rename tar subdirectory ' + source 
                                    + ' to ' + dir + '\nError was:' + str(e))
            else:
                raise Exception('Given subpath "' + source + '" does not exist '
                                ' in the tar package!')
            try:
                shutil.rmtree(target)
            except Exception, e:
                raise Exception('Failed to remove unnecessary tar structure "'
                                + target + '"\nError was:' + str(e))

        os.unlink(pkg)

        return result

    def sync(self, base):

        self.supported()

        self.delete(base)

        self.add(base)

    def supported(self):
        
        return Overlay.supported(self, [(self.binary,  'tar', 'app-arch/tar'),])

if __name__ == '__main__':
    import doctest, sys

    # Ignore warnings here. We are just testing
    from warnings     import filterwarnings, resetwarnings
    filterwarnings('ignore')

    doctest.testmod(sys.modules[__name__])

    resetwarnings()
